var MouthPhysicsSprite = cc.PhysicsSprite.extend({
    space: null,
    mouseBody: null,
    pivotJoint: null,
    ctor: function (dir, name) {
        this._super(dir + name);
        this.registerEvent();
    },
    registerEvent: function () {
        this.cardTouchListener = cc.EventListener.create({
            event: cc.EventListener.TOUCH_ONE_BY_ONE,
            swallowTouches: true,
            onTouchBegan: this.onTouchBegan,
            onTouchMoved: this.onTouchMoved,
            onTouchEnded: this.onTouchEnded,
            onTouchCancelled: this.onTouchCancelled,
        });
        cc.eventManager.addListener(this.cardTouchListener, this);
    },
    onTouchBegan: function (touch, event) {
        var target = event.getCurrentTarget();
        var position = touch.getLocation();
        var locationInNode = target.convertToNodeSpace(position);
        var size = target.getContentSize();
        var rect = cc.rect(0, 0, size.width, size.height);
        if (!cc.rectContainsPoint(rect, locationInNode)) {
            return false;
        }

        var mouseBody = new cp.StaticBody();
        mouseBody.setPos(cp.v(position.x, position.y));
        var pivot = new cp.PivotJoint(target.getBody(), mouseBody, cp.v(position.x, position.y));
        var gear = new cp.RatchetJoint(target.getBody(), mouseBody, 0, 1);
        target.space.addConstraint(pivot);
        target.space.addConstraint(gear);
        target.mouseBody = mouseBody;
        target.pivotJoint = pivot;
        target.gearJoint = gear;
        return true;
    },
    onTouchMoved: function (touch, event) {
        var target = event.getCurrentTarget();
        if (target.mouseBody) {
            cc.log('moved : 2');
            var position = touch.getLocation();
            target.mouseBody.setPos(cp.v(position.x, position.y));
        }
    },
    onTouchEnded: function (touch, event) {
        var target = event.getCurrentTarget();
        cc.log('ended : 1');
        if (target.mouseBody) {
            cc.log('ended : 2');
            target.space.removeConstraint(target.pivotJoint);
            target.space.removeConstraint(target.gearJoint);
        }
    },
    onTouchCancelled: function (touch, event) {
        var target = event.getCurrentTarget();
        target.onTouchEnded(touch, event);
    }
});

var TestLayer_Fishing = cc.Layer.extend({
    ctor: function () {
        this._super();

        this.loadPhysicsSpace();
        this.loadDebugPhysicsSpace();
        // this.loadSpriteBody();
        // this.loadTapBody();
        this.loadFishingLine();
    },
    loadPhysicsSpace: function () {
        this.space = new cp.Space();
        this.space.gravity = cp.v(0, -350);
        this.space.sleepTimeThreshold = 0.5;
        this.space.damping = 0.2;

        var thickness = 100;
        var staticBody = this.space.staticBody;
        var w = cc.winSize.width;
        var h = cc.winSize.height;
        var walls = [
            new cp.SegmentShape(staticBody, cp.v(-thickness, -thickness), cp.v(w, -thickness), thickness),
            new cp.SegmentShape(staticBody, cp.v(-thickness, -thickness), cp.v(-thickness, h), thickness),
            new cp.SegmentShape(staticBody, cp.v(w + thickness, -thickness), cp.v(w + thickness, h), thickness),
            new cp.SegmentShape(staticBody, cp.v(-thickness, h + thickness), cp.v(w, h + thickness), thickness),
        ];
        for (var i = 0; i < walls.length; i++) {
            var shape = walls[i];
            shape.setElasticity(0.5);
            shape.setFriction(0);
            this.space.addStaticShape(shape);
        }

        // this.setScale(0.8);

        var timeRemain = 0;
        var stepTime = 1 / 60.0;
        this.update = function (dt) {
            timeRemain += dt;
            while (timeRemain - stepTime >= 0) {
                this.space.step(stepTime);
                timeRemain -= stepTime;
            }
        };
        this.scheduleUpdate();
    },
    loadDebugPhysicsSpace: function () {
        var node = new cc.PhysicsDebugNode(this.space);
        this.addChild(node, Number.MAX_VALUE);
    },
    loadSpriteBody: function () {
        var node = new cc.PhysicsSprite("res/z_test_fish/fishing_rod.png");
        this.addChild(node);

        var mess = 10;
        var body = new cp.Body(mess, cp.momentForBox(mess, node.width, node.height));
        this.space.addBody(body);
        body.setPos(cp.v(cc.winSize.width / 2, cc.winSize.height / 2));

        var shape = new cp.BoxShape(body, node.width, node.height);
        this.space.addShape(shape);
        node.setBody(body);
        shape.setElasticity(0.5);
        shape.setFriction(0.5);
    },
    loadTapBody: function () {
        var node = new MouthPhysicsSprite('res/z_test_fish/', 'fishing_rod.png');
        node.space = this.space;
        this.addChild(node);

        var mess = 1;
        var body = new cp.Body(mess, cp.momentForBox(mess, node.width, node.height));
        body.setPos(cp.v(cc.winSize.width / 2, cc.winSize.height / 2));
        body.sprite = node;
        this.space.addBody(body);

        var shape = new cp.BoxShape(body, node.width, node.height);
        shape.setElasticity(0.5);
        shape.setFriction(0.5);
        this.space.addShape(shape);

        node.setBody(body);
    },
    loadFishingLine: function () {
        var w = cc.winSize.width;
        var h = cc.winSize.height;
        var headBody = new cp.StaticBody();
        headBody.setPos(cp.v(w / 2, h));
        var HeadSize = 50;
        var HeadShape = new cp.BoxShape(headBody, HeadSize, HeadSize);
        this.space.addStaticShape(HeadShape);

        var ropeNodeMess = 10;
        var ropeNodeRadius = 5;
        var ropeNodeCount = 1;
        var stiffness = 3000;
        var lastBody = null;
        for (var i = 0; i < ropeNodeCount; i++) {
            var body = new cp.Body(1, cp.momentForCircle(ropeNodeMess, 0, ropeNodeRadius, cp.v(0, 0)));
            // body.setPos(cp.v(w / 2, headBody.getPos().y - HeadSize / 2 - ropeNodeRadius * (2 * i + 1)));
            body.setPos(cp.v(w / 2, headBody.getPos().y - (HeadSize / 2 + ropeNodeRadius * (2 * i + 1)) + 100));
            this.space.addBody(body);
            var shape = new cp.CircleShape(body, ropeNodeRadius, cp.v(0, 0));
            this.space.addShape(shape);
            if (i == 0) {
                // var spring = new cp.DampedSpring(headBody, body, cp.v(0, 0), cp.v(0, 0), HeadSize / 2 + ropeNodeRadius*1.1, stiffness, 0.0);
                // this.space.addConstraint(spring);
            }
            else {
                var spring = new cp.DampedSpring(lastBody, body, cp.v(0, 0), cp.v(0, 0), ropeNodeRadius * 2 + 100, stiffness, 0.0);
                this.space.addConstraint(spring);
            }
            lastBody = body;
        }

        // var tailNode = new MouthPhysicsSprite('res/slider/', 'btn.png');
        // tailNode.space = this.space;
        // this.addChild(tailNode);
        // var tailBody = new cp.Body(ropeNodeMess, cp.momentForBox(ropeNodeMess, tailNode.widget, tailNode.height));
        // tailBody.setPos(cp.v(lastBody.x, 0));
        // tailBody.sprite = tailNode;
        // this.space.addBody(tailBody);
        // var tailShape = new cp.BoxShape(tailBody, tailNode.widget, tailNode.height);
        // this.space.addShape(tailShape);
        // tailNode.setBody(tailBody);


        var tailNode = new MouthPhysicsSprite('res/z_test_slider/', 'btn.png');
        tailNode.space = this.space;
        this.addChild(tailNode);
        var mess = 1;
        var body = new cp.Body(mess, cp.momentForBox(mess, tailNode.width, tailNode.height));
        body.setPos(cp.v(w / 2, h / 2));
        body.sprite = tailNode;
        this.space.addBody(body);
        var shape = new cp.BoxShape(body, tailNode.width, tailNode.height);
        shape.setElasticity(0.5);
        shape.setFriction(0.5);
        this.space.addShape(shape);
        tailNode.setBody(body);
        var spring = new cp.DampedSpring(lastBody, body, cp.v(0, 0), cp.v(0, 0), ropeNodeRadius + tailNode.height / 2 , stiffness, 0.0);
        this.space.addConstraint(spring);
    }
});

var TestScene_Fishing = cc.Scene.extend({
    ctor: function () {
        this._super();

        var layer = new TestLayer_Fishing();
        this.addChild(layer);
    }
});